#!/usr/bin/perl

package MATT::FreeBSD;

##
# FreeBSD.pm by Matt Simerson <matt@cadillac.net>
##

##
# ChangeLog
##
# 0.1.0 - 05.23.2003 - Created module from code I wrote for other apps

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
require Exporter;

@ISA     = qw(Exporter);
@EXPORT  = qw(InstallPort InstallPackage);
$VERSION = 0.2.0;

use MATT::Utility;

sub InstallPort($$;$$)
{
	my ($name, $base, $booger, $check) = @_;
	my ($dir);

	if ($booger)
	{
		$dir = "/usr/ports/$base/$booger";
	} else {
		$dir = "/usr/ports/$base/$name";
	};

	CheckPortsAge("30");

	if (! $check) {
		$check = $name;
	};

	my $pkg_info = FindTheBin("pkg_info");
	my $grep     = FindTheBin("grep");

	my $r = `$pkg_info | cut -d" " -f1 | $grep $check-`; chomp $r;
	if ( ! $r ) {
		$r = `$pkg_info | cut -d" " -f1 | $grep $check`; chomp $r;
	};

	if ( $r ne "" )
	{
		printf "InstallPort: %-20s installed as (%s).\n", $name, $r;
		return 0;
	} else {
		print "InstallPort: installing $name...";
		chdir("$dir") or die "couldn't cd to $dir: $!\n";
		if ( $name eq "qmail" ) {
			SysCmd( "make enable-qmail clean");
			move("/usr/local/etc/rc.d/qmail.sh", "/usr/local/etc/rc.d/qmail.sh-dist") or die "$!";
		} elsif ( $name eq "ezmlm-idx" ) {
			SysCmd( "make -DWITH_MYSQL install");
			copy("work/ezmlm-0.53/ezmlmrc", "/usr/local/bin");
			SysCmd( "make clean");
		} elsif ( $name eq "mod_php4" ) {
			SysCmd( "make -DWITH_APACHE2 install");
			SysCmd( "make clean");
		} elsif ( $name eq "squirrelmail" ) {
			my $ans = YesOrNo("Are you using Apache2?");
			if ($ans) {
				SysCmd( "make -DWITH_APACHE2 install");
			} else {
				SysCmd( "make install");
			};
			SysCmd( "make clean");
		} else {
			SysCmd( "make install clean");
		};
		print "done.\n";
		return 1;
	};
};

sub CheckPortsAge
{
	my ($age) = @_;

	my $dl_site   = "http://matt.simerson.net";
	my $toaster   = "$dl_site/computing/mail/toaster";       

	if ( -M "/usr/ports" > $age )
	{
	print "\n\nCheckPortsAge: CAUTION: You might want to update your ports tree!\n\n";

	my $answer = YesOrNo( "\n\nWould you like me to do it for you?:");
	if ( $answer )
	{
		my $cvsupbin = FindTheBin("cvsup");

		if (!$cvsupbin) { InstallPackage("cvsup-without-gui"); };

		my $cvsup = "/etc/cvsup-ports";

		if ( !-e $cvsup )
		{
			FetchFile("$toaster/cvsup-ports");
			move("cvsup-ports", $cvsup);
		};
			SysCmd("$cvsupbin -g $cvsup");
		};
	};
};

sub InstallPackage($;$)
{   
	my ($package, $alt) = @_;
	my $r;  

	my $pkg_info = FindTheBin("pkg_info");
	my $pkg_add  = FindTheBin("pkg_add");
	my $grep     = FindTheBin("grep");

	if ( $alt )
	{
		$r = `$pkg_info | cut -d" " -f1 | $grep $alt-`;
		if ( ! $r ) {
			$r = `$pkg_info | cut -d" " -f1 | $grep $alt`;
		};
	} else {
		$r = `$pkg_info | cut -d" " -f1 | $grep $package-`;
		if ( ! $r ) {
			$r = `$pkg_info | cut -d" " -f1 | $grep $package`;
		};
	};
	chomp $r;

	if ( $r )
	{
		printf "InstallPackage: %-20s installed as (%s).\n", $package, $r;
	} else {
		print "InstallPackage: installing $package....";
#		$ENV{"PACKAGESITE"} = $pk_url;
		SysCmd("$pkg_add -r $package");
		print "done.\n";
	};
};

1;
__END__

=head1 NAME

        MATT::FreeBSD.pm - FreeBSD management functions

=head1 SYNOPSIS

        FreeBSD.pm is a grouping of frequently used functions I've written
        for working with FreeBSD systems.

=head1 DESCRIPTION

=head1 AUTHOR

        Matt Simerson <matt@cadillac.net>

=head1 BUGS

        None known. Report any to matt@cadillac.net.

=head1 TODO

        Needs more documentation.

=head1 SEE ALSO

        http://matt.simerson.net/computing/

=head1 COPYRIGHT

        Copyright 2003, Matt Simerson. All Right Reserved.

=cut
