#!/usr/bin/perl

package MATT::Utility;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use Exporter;

@ISA   = qw(Exporter);
@EXPORT = qw(SysCmd FindTheBin ReadFile WriteFile StripLastDirFromPath 
	ArchiveFile YesOrNo GetDirFiles CleanTmpDir GetTheDate GetAnswer
);
@EXPORT_OK = qw();
$VERSION = 0.6.1;

sub YesOrNo
{
	my $ans;
	do
	{
		print "(y/n): ";
		$ans = <STDIN>; chomp($ans);  
		$ans = lc($ans); 
	} until ( $ans eq "n" || $ans eq "y" );

	if ($ans eq "y") { return 1; };
	if ($ans eq "n") { return 0; };
};

sub StripLastDirFromPath($) 
{
=head2
	takes a path like "/usr/home/matt" and returns "/usr/home" and "matt"
=cut

	my ($dir) = @_;

	if ( $dir =~ /\/$/ ) { chop $dir };

	my $rindex = rindex($dir, "/");
	my $updir  = substr($dir, 0, $rindex);
	$rindex++;
	my $curdir = substr($dir, $rindex);       

	return $updir, $curdir;
};

sub WriteFile($@)
{
	my ($file, @lines) = @_;

	open FILE, ">$file" or die "WriteFile: couldn't open: $!";
		foreach my $line (@lines)
		{
			print FILE "$line\n";
		};
	close FILE;
};

sub ReadFile($)
{
	my ($file) = @_;

	if ( ! -r $file ) {
		if ( -e $file ) {
			warn "ReadFile: $file is not readable!\n";
			return 0;
		};
	};

	open(FILE, $file) or warn "ReadFile: couldn't open $file: $!";
		my @lines = <FILE>;
	close FILE;

	chomp @lines;
	return @lines;
};

sub FindTheBin($)
{
	my ($bin) = @_;

	my $prefix = "/usr/local";
  
	if    ( -x "$prefix/bin/$bin"       ) { return "/usr/local/bin/$bin";  }
	elsif ( -x "$prefix/sbin/$bin"      ) { return "/usr/local/sbin/$bin"; }
	elsif ( -x "$prefix/mysql/bin/$bin" ) { return "$prefix/mysql/bin/$bin"; }
	elsif ( -x "/bin/$bin"              ) { return "/bin/$bin";            }
	elsif ( -x "/usr/bin/$bin"          ) { return "/usr/bin/$bin";        }
	elsif ( -x "/sbin/$bin"             ) { return "/sbin/$bin";           }
	elsif ( -x "/usr/sbin/$bin"         ) { return "/usr/sbin/$bin";       }
	else  { return 0; };
};

sub SysCmd($)
{
	my ($cmd) = @_;

	my $r = system $cmd;
	if ($r != 0)
	{
		print "SysCmd: $cmd\n";
		print "SysCmd: result: $r\n";
	};    
	return $r;
};

sub ArchiveFile($)
{
	my ($file) = @_;

	my $date = time;

	my $cmd = "sudo cp $file $file.$date";
	SysCmd($cmd);

#	use File::Copy;
#	copy($file, "$file\.$date") or die "backup of $file to $file.$date failed: $!\n";

	print "$file backed up to $file.$date\n";
};

sub GetDirFiles($)
{
	my ($dir) = @_;
	my @files;

	opendir(D, $dir) or die "GetDirFiles: couldn't open $dir: $!";
	while( defined ( my $f = readdir(D) ) )
	{
		next if $f =~ /^\.\.?$/;
		push @files, "$dir/$f";
	};
	closedir(D);
	return @files;
};

sub CleanTmpDir($)
{
	my ($dir) = @_;

	my @list = GetDirFiles($dir);
	chdir ($dir);
	foreach my $e (@list)
	{
		if ( -f $e )
		{
		DeleteFile($e);
		}
		elsif ( -d $e )
		{
			use File::Path;
			rmtree $e or die "CleanTmpdir: couldn't delete $e\n";
		};
	};
};

sub DeleteFile($;$)
{       
	my ($file, $warn) = @_;

	if ($warn)    
	{
		unlink $file or warn "WARNING: couldn't delete $file: $!\n";
	} else {
		unlink $file or die "FATAL: couldn't delete $file: $!\n";
	};
};

sub GetTheDate(;$$)
{
	my ($bump, $debug) = @_;

BEGIN {
	eval { require Date::Format };
	if ($@) {
		die "\nERROR: Date::Format is not installed. Try this:\n
perl -MCPAN -e shell
install Date::Format
";
	} else {
		Date::Format->import();
	};
};

	if ($bump) 
	{
		my $ss = time2str("%S", (time      ));
		my $mn = time2str("%M", ( time                  ));
		my $hh = time2str("%H", ( time - (       $bump) ));
		my $dd = time2str("%d", ( time - (       $bump) ));
		my $mm = time2str("%m", ( time - (       $bump) ));
		my $yy = time2str("%Y", ( time - (       $bump) ));
		my $lm = time2str("%m", ( time - (2592000+$bump)));

		print "GetTheDate: $yy/$mm/$dd $hh:$mn\n" if $debug;
	}
	else
	{
		my $ss = time2str("%S", (time      ));
		my $mn = time2str("%M", (time      ));
		my $hh = time2str("%H", (time      ));
		my $dd = time2str("%d", (time      ));
		my $mm = time2str("%m", (time      ));
		my $yy = time2str("%Y", (time      ));
		my $lm = time2str("%m", (time-2592000));
		print "GetTheDate: $yy/$mm/$dd $hh:$mn\n" if $debug;
		return $dd, $mm, $yy, $lm, $hh, $mn, $ss;
	};
};

sub GetAnswer
{
	my ($q, $default) = @_;
  
	if ($default) { print "Please enter $q. ($default) :" }
	else          { print "Please enter $q: "             };

	my $ans = <STDIN>;
	chomp $ans;

	if ($ans ne "") { return $ans; }
	else
	{
		if ($default) { return $default }
		else          { return 0;       }
	};
};

1;
__END__


=head1 NAME

        MATT::Utility.pm - Common functions

=head1 SYNOPSIS

        Utility.pm is a grouping of frequently used functions I've written.

=head1 DESCRIPTION


=head1 AUTHOR

        Matt Simerson <matt@cadillac.net>

=head1 BUGS

        None known. Report any to author.

=head1 TODO

=head1 SEE ALSO

        http://matt.simerson.net/computing/

=head1 COPYRIGHT

        Copyright 2003, Matt Simerson. All Right Reserved.

=cut
